@@ -61,7 +61,7 @@ struct lxc_arguments {
 	const char *share_ns[32]; // size must be greater than LXC_NS_MAX
 
 	/* for lxc-console */
-	int ttynum;
+	unsigned int ttynum;
 	char escape;
 
 	/* for lxc-wait */
@@ -161,8 +161,6 @@ extern int lxc_arguments_parse(struct lxc_arguments *args,
 
 extern int lxc_arguments_str_to_int(struct lxc_arguments *args, const char *str);
 
-extern const char *lxc_strerror(int errnum);
-
 #define lxc_error(arg, fmt, args...) if (!(arg)->quiet)			\
 	fprintf(stderr, "%s: " fmt "\n", (arg)->progname,  ## args)
 
